function [Mellin,beta,Q,Kmin,Nb,Mmin,elog,yyelog]=mellin_trans(yy,r,V1,Nb,Q,dispflag);

% Discrete Mellin transform
% [Mellin,beta,Q,Kmin,Nb,Mmin,elog,yyelog]=mellin_trans(yy,r,V1,Nb,Q,dispflag);
%
% Algorithm from Bertrand, Bertrand and Ovarlez
% INPUT	yy = analysed signal (regular sampling, sampling period set to 1)
%	r = amplitude parameter (take r=-H-1)
%	V1 = time of the 1st sample (yy[1]=signal(V1)) (default=1)
%       Nb = number of points for beta (opt.)
%	Q = coefficient for sampling step of the Mellin transform (opt.)
%	dispflag = set to 'n' to skip warnings (opt.)
%
% OUTPUT Mellin = discrete Mellin transform of yy
%	beta = Mellin scale variable vector
%	Q = 1/log(Q) is the sampling step for Mellin scale variable	
%	Kmin = first index of the sampled beta's
%	Nb = number of points for beta
%	Mmin =  min. index for the geometrical sampling of yy (in q^n)
%	elog = geometrical sampling vector used
% 	yyelog = geometrically resampled yy
%
% This is a non interactive version of Mellin transform.
% Note that the signal (in yyelog) is completed by
% zero-padding if the required sampling leaves the
% interval of definition of yy, assumed to be [V1:V1+length(yy)-1]
%
% Example:
% [mellin,beta,Q,K,Nb,M]=mellin_trans(yy,r,1) ;
%
% pborgnat (february 2000)
%

%% Commentaires en franais:
%% Transformee de Mellin discrete
%Algorithme selon Bertrand et al.
%INPUT: yy = signal a analyser
%       r = parametre du groupe de dilatation
%       V1 = debut du signal echantillonne fourni (en dt=1 !) (yy[1] = signal(V1))
%       Q = pas d'echantillonnage pour Mellin
%       dispflag = 'n' enleve les warning
%OUTPUT: Mellin = transf de Mellin discrete
%        beta = variable de Mellin echantillonnee
%        Q = rapport de periodisation par dilatation ; pas en Mellin =  1/ln(Q)
%        Kmin = indice min des beta echantillonnes
%        Nb = nombre de points en beta
%        Mmin = indice min de l'echantillonnage log (en q^n)
%        elog = vecteur des Nb pts d'echantillonnage log
%        yyelog = signal reechantillonne geometriquement
%
%Version non interactive.
%Convention: echantillon 1 = valeur au temps V1.
%Rq: si l'ech. depasse la zone de definition du signal,
% completion par des zeros.
%exemple: [mellin,beta,Q,K,Nb,M]=mellin_trans(yy,r,1) ;
%
%(pborgnat fevrier 2000)


if nargin<6, dispflag='y' ; end ; 

[yrow,ycol] = size(yy) ;
if yrow ~= 1, yy = yy' ; end ;
N = length(yy) ;
%% N est le nbre de points en temps du signal entre %%

%% Choix des parametres d'echantillonnage

%% procedure pour trouver [F1 F2] support de YY... => DF %%
%% et [V1 V2] support de yy... => Q_min %%
%approche simplifiee ; on utilise la variable temporelle v = t/\tau_e 
if nargin==2,
V1 = 1 ;
end;

V0 = 1-V1 ; %indice, evt non entier, de la position du zero en temps.
Ny = N ;
% Ny nbre de points utilises dans l'analyse 
V2 = Ny+V1-1 ; %borne max du support temporel ;
Ny2 = Ny*2 ;  %longueur du signal unilateral a analyser ;
%%ou pour un signal unilateral => borne max temporelle du signal a analyser =
%Ny2 = Ny*2 ;
%V2 = Ny2+V1-1 ;
if nargin<5, 
  Q = V2/V1 ;
else
  if Q<(V2/V1),
  warnQ = ['Warning, Q is too small (Qmin = ',num2str(V2/V1),').'] ;
  if dispflag=='n', warnQ = []; end ;
    disp(warnQ) ;
  end ;
  V2 = V1*Q ;   %V2 effectif de l'analyse
end ;

fyy = yy ;
%fyy = [yy(1:Ny) zeros(1,Ny)] ;
% interpretation freq. : prendre fftshift(fyy) %%
% interpretation temporelle : on rajoute des 0 aux temps negatifs => unilateral %%
%YY = ifft(fyy)*Ny2 ;
%% YY est analytique et fft(fyy) = Ny2*fliplr(YY)([Ny2 1:Ny2-1]) %%
%% inversement : FT(YY) = [zeros(1,V2) yy] *Ny2 ;

%choix de DF
fmin=0. ; fmax=0.5 ;
DF = fmax-fmin ;

%Q_min = V2/V1 ;
Nb_min = fix(V2*DF*log(Q)) ;
 %% condition de bon echantillonnage pour Mellin discrete
Nbsp = 2^nextpow2(Nb_min) ;
%% choix de Nb
if nargin<4,
  Nb = Nbsp ;
else, 
  if Nb<Nb_min, 
    warnNb = ['Warning, Nb is to small (Nbmin = ',num2str(Nb_min),').'] ;
    if dispflag=='n', warnNb = [] ; end ;
    disp(warnNb) ;
  end ;
end ;

% d'ou les parametres d'echantillonnage
beta0 = Nb/2/log(Q) ;
q = Q.^(1./Nb) ;  %% NE PAS UTILISER ; PB d'arrondi numerique !!!
logq = log(Q)/Nb ;
Mmin = ceil(log(V1)/logq) ;

%% echantillonnage logarithmique en q^n
nn = (1:Nb) + Mmin-1 ;
elog = exp(logq.*nn) ; % instants q^nn 

% methode par interpolation
eind_sup=ceil(elog-V1+1);   %indices
eind_inf=fix(elog-V1+1);
einf = fix(elog);  %vrai instant
emax = max(eind_sup) ;
if emax>Ny,
  fyy((Ny+1):emax) = zeros(1,emax-Ny) ;
end ;
yyelog=(fyy(eind_sup)-fyy(eind_inf)).*(elog-einf)+fyy(eind_inf);
%yyelog=yyelog/mellin_NormR(yyelog,elog,r)*mellin_NormR(yy,V1:V2,r) ;
% normalisation de telle sorte que la serie geom ait la meme norme que depart
 
% methode par reconstruction en Fourier... (a faire) 

%%echantillonnage lineaire en beta
Kmin = ceil(-beta0*log(Q)) ;
k = (0:Nb-1) + Kmin*ones(1,Nb) ;
beta = k/log(Q) ;

%% transformee discrete de mellin
mm=1:Nb ;
ampl = (r+1)*logq ;
cor_phase = (2*pi*i*Kmin/Nb) ;
blah = exp(mm*(cor_phase+ ampl)-cor_phase).*yyelog ;
TFblah = Nb*logq*exp((Mmin-1)*ampl)*(ifft(blah)) ;
Mellin = TFblah.*exp(2*pi*i*Mmin*k/Nb) ; 


%% Verifier la normalisation
V = linspace(V1,V1+Ny-1,Ny) ;
expnorm = 2*r+1 ;
%normR_sig2 = (yy)*(yy.*V.^expnorm)' ;
normR_sig2=mellin_NormR(yyelog,elog,r).^2; 
%normalisation pour retrouver les amplitudes (et non norme) par reconstruction.
Mellin = sqrt(normR_sig2)*logq*sqrt(log(Q))/norm(Mellin)*Mellin ;
  

